# -*- coding: utf-8 -*- #
# frozen_string_literal: true

# DO NOT EDIT
# This file is automatically generated by `rake builtins:viml`.
# See tasks/builtins/viml.rake for more info.

module Rouge
  module Lexers
    class VimL
      KEYWORDS = {}.tap do |kw|
          kw[:command] = Set.new ["a", "al", "all", "ar", "arga", "argadd", "argd", "argded", "argdedupe", "argdelete", "arge", "argedit", "argg", "argglobal", "argl", "arglocal", "args", "argu", "argument", "as", "ascii", "b", "bN", "bNext", "ba", "bad", "badd", "ball", "balt", "bd", "bdelete", "bf", "bfirst", "bl", "blast", "bm", "bmodified", "bn", "bnext", "bp", "bprevious", "br", "brea", "break", "brewind", "buffer", "buffers", "bun", "bunload", "bw", "bwipeout", "cN", "cNext", "cNf", "cNfile", "cabo", "cabove", "cad", "caddbuffer", "cadde", "caddexpr", "caddf", "caddfile", "caf", "cafter", "cb", "cbe", "cbefore", "cbel", "cbelow", "cbo", "cbottom", "cbuffer", "cc", "ccl", "cclose", "ce", "center", "cex", "cexpr", "cf", "cfile", "cfir", "cfirst", "cg", "cgetb", "cgetbuffer", "cgete", "cgetexpr", "cgetfile", "changes", "che", "checkpath", "checkt", "checktime", "chi", "chistory", "cl", "cla", "clast", "cle", "clearjumps", "clip", "clipreset", "clist", "clo", "close", "cn", "cnew", "cnewer", "cnext", "cnf", "cnfile", "col", "colder", "colo", "colorscheme", "comc", "comclear", "comp", "compiler", "con", "continue", "cope", "copen", "cp", "cpf", "cpfile", "cprevious", "cq", "cquit", "cr", "crewind", "cs", "cscope", "cst", "cstag", "cw", "cwindow", "defc", "defcompile", "delel", "delep", "deletel", "deletep", "deletl", "deletp", "dell", "delm", "delmarks", "delp", "dep", "di", "dif", "diffg", "diffget", "diffo", "diffoff", "diffp", "diffpatch", "diffpu", "diffput", "diffs", "diffsplit", "difft", "diffthis", "diffupdate", "dig", "digraphs", "disa", "disassemble", "display", "dj", "djump", "dl", "dli", "dlist", "dp", "dr", "drop", "ds", "dsearch", "dsp", "dsplit", "e", "ea", "earlier", "edit", "em", "emenu", "endfo", "endfor", "endt", "endtry", "endw", "endwhile", "ene", "enew", "ex", "exi", "exit", "exu", "exusage", "f", "file", "files", "fin", "fina", "finally", "find", "fini", "finish", "fir", "first", "fix", "fixdel", "fo", "fold", "foldc", "foldclose", "foldo", "foldopen", "g", "global", "go", "goto", "gu", "gui", "gv", "gvim", "ha", "hardcopy", "helpc", "helpclose", "helpf", "helpfind", "helpt", "helptags", "i", "ij", "ijump", "il", "ilist", "int", "intro", "ip", "iput", "is", "isearch", "isp", "isplit", "ju", "jumps", "l", "lN", "lNext", "lNf", "lNfile", "la", "lab", "labove", "lad", "laddb", "laddbuffer", "laddexpr", "laddf", "laddfile", "laf", "lafter", "last", "lat", "later", "lb", "lbe", "lbefore", "lbel", "lbelow", "lbo", "lbottom", "lbuffer", "lcl", "lclose", "lcs", "lcscope", "le", "left", "lex", "lexpr", "lf", "lfile", "lfir", "lfirst", "lg", "lgetb", "lgetbuffer", "lgete", "lgetexpr", "lgetfile", "lhi", "lhistory", "list", "ll", "lla", "llast", "lli", "llist", "lmak", "lmake", "lne", "lnew", "lnewer", "lnext", "lnf", "lnfile", "lo", "loadview", "lockv", "lockvar", "lol", "lolder", "lop", "lopen", "lp", "lpf", "lpfile", "lprevious", "lr", "lrewind", "ls", "lt", "ltag", "lw", "lwindow", "m", "marks", "mes", "messages", "mk", "mkexrc", "mks", "mksession", "mksp", "mkspell", "mkv", "mkvie", "mkview", "mkvimrc", "mod", "mode", "move", "n", "nb", "nbc", "nbclose", "nbkey", "nbs", "nbstart", "next", "nextgroup", "noh", "nohlsearch", "nu", "number", "o", "ol", "oldfiles", "on", "only", "open", "opt", "options", "ow", "ownsyntax", "p", "pa", "packadd", "packl", "packloadall", "pb", "pbuffer", "pc", "pclose", "ped", "pedit", "po", "pop", "pp", "ppop", "pre", "preserve", "prev", "previous", "print", "ps", "psearch", "pt", "ptN", "ptNext", "ptag", "ptf", "ptfirst", "ptj", "ptjump", "ptl", "ptlast", "ptn", "ptnext", "ptp", "ptprevious", "ptr", "ptrewind", "pts", "ptselect", "pu", "put", "pw", "pwd", "q", "qa", "qall", "quit", "quita", "quitall", "r", "read", "rec", "recover", "red", "redo", "redr", "redraw", "redraws", "redrawstatus", "redrawt", "redrawtabline", "redrawtabp", "redrawtabpanel", "reg", "registers", "res", "resize", "ret", "retab", "rew", "rewind", "ri", "right", "ru", "rund", "rundo", "runtime", "rv", "rviminfo", "sN", "sNext", "sa", "sal", "sall", "sargument", "sav", "saveas", "sb", "sbN", "sbNext", "sba", "sball", "sbf", "sbfirst", "sbl", "sblast", "sbm", "sbmodified", "sbn", "sbnext", "sbp", "sbprevious", "sbr", "sbrewind", "sbuffer", "scr", "scripte", "scriptencoding", "scriptnames", "scriptv", "scriptversion", "scs", "scscope", "setf", "setfiletype", "sf", "sfind", "sfir", "sfirst", "sh", "shell", "sig", "sign", "sim", "simalt", "sla", "slast", "smi", "smile", "sn", "snext", "so", "source", "spe", "spelld", "spelldump", "spellgood", "spelli", "spellinfo", "spellr", "spellra", "spellrare", "spellrepall", "spellu", "spellundo", "spellw", "spellwrong", "spr", "sprevious", "sre", "srewind", "st", "sta", "stag", "star", "startg", "startgreplace", "startinsert", "startr", "startreplace", "stj", "stjump", "stop", "stopi", "stopinsert", "sts", "stselect", "sun", "sunhide", "sus", "suspend", "sv", "sview", "sync", "syncbind", "t", "tN", "tNext", "ta", "tabN", "tabNext", "tabc", "tabclose", "tabe", "tabedit", "tabf", "tabfind", "tabfir", "tabfirst", "tabl", "tablast", "tabm", "tabmove", "tabn", "tabnew", "tabnext", "tabo", "tabonly", "tabp", "tabprevious", "tabr", "tabrewind", "tabs", "tag", "tags", "te", "tearoff", "tf", "tfirst", "tj", "tjump", "tl", "tlast", "tn", "tnext", "tp", "tprevious", "tr", "trewind", "try", "ts", "tselect", "u", "undo", "undoj", "undojoin", "undol", "undolist", "unh", "unhide", "up", "update", "v", "ve", "version", "vglobal", "vi", "vie", "view", "vimBang", "visual", "viu", "viusage", "vne", "vnew", "vs", "vsplit", "w", "wN", "wNext", "wa", "wall", "wi", "winp", "winpos", "winsize", "wl", "wlrestore", "wn", "wnext", "wp", "wprevious", "wq", "wqa", "wqall", "write", "wu", "wundo", "wv", "wviminfo", "x", "xa", "xall", "xit", "xr", "xrestore", "y", "yank", "z"]
          kw[:option] = Set.new ["ac", "acd", "acl", "act", "ai", "al", "aleph", "allowrevins", "ambiwidth", "ambw", "ar", "arab", "arabic", "arabicshape", "ari", "arshape", "asd", "autochdir", "autocomplete", "autocompletedelay", "autocompletetimeout", "autoindent", "autoread", "autoshelldir", "autowrite", "autowriteall", "aw", "awa", "background", "backspace", "backup", "backupcopy", "backupdir", "backupext", "backupskip", "balloondelay", "ballooneval", "balloonevalterm", "balloonexpr", "bdir", "bdlay", "belloff", "beval", "bevalterm", "bex", "bexpr", "bg", "bh", "bin", "binary", "bk", "bkc", "bl", "bo", "bomb", "breakat", "breakindent", "breakindentopt", "bri", "briopt", "brk", "browsedir", "bs", "bsdir", "bsk", "bt", "bufhidden", "buflisted", "buftype", "casemap", "cb", "cc", "ccv", "cd", "cdh", "cdhome", "cdpath", "cedit", "cf", "cfu", "ch", "charconvert", "chi", "chistory", "ci", "cia", "cin", "cindent", "cink", "cinkeys", "cino", "cinoptions", "cinscopedecls", "cinsd", "cinw", "cinwords", "clipboard", "clipmethod", "cm", "cmdheight", "cmdwinheight", "cmp", "cms", "co", "cocu", "cole", "colorcolumn", "columns", "com", "comments", "commentstring", "compatible", "complete", "completefunc", "completeitemalign", "completeopt", "completepopup", "completeslash", "completetimeout", "concealcursor", "conceallevel", "confirm", "copyindent", "cot", "cp", "cpm", "cpo", "cpoptions", "cpp", "cpt", "crb", "cryptmethod", "cscopepathcomp", "cscopeprg", "cscopequickfix", "cscoperelative", "cscopetag", "cscopetagorder", "cscopeverbose", "csl", "cspc", "csprg", "csqf", "csre", "cst", "csto", "csverb", "cto", "cuc", "cul", "culopt", "cursorbind", "cursorcolumn", "cursorline", "cursorlineopt", "cwh", "debug", "deco", "def", "define", "delcombine", "dex", "dg", "dia", "dict", "dictionary", "diff", "diffanchors", "diffexpr", "diffopt", "digraph", "dip", "dir", "directory", "display", "dy", "ea", "ead", "eadirection", "eb", "ed", "edcompatible", "ef", "efm", "ei", "eiw", "ek", "emo", "emoji", "enc", "encoding", "endoffile", "endofline", "eof", "eol", "ep", "equalalways", "equalprg", "errorbells", "errorfile", "errorformat", "esckeys", "et", "eventignore", "eventignorewin", "ex", "expandtab", "exrc", "fcl", "fcs", "fdc", "fde", "fdi", "fdl", "fdls", "fdm", "fdn", "fdo", "fdt", "fen", "fenc", "fencs", "fex", "ff", "ffs", "ffu", "fic", "fileencoding", "fileencodings", "fileformat", "fileformats", "fileignorecase", "filetype", "fillchars", "findfunc", "fixendofline", "fixeol", "flp", "fml", "fmr", "fo", "foldclose", "foldcolumn", "foldenable", "foldexpr", "foldignore", "foldlevel", "foldlevelstart", "foldmarker", "foldmethod", "foldminlines", "foldnestmax", "foldopen", "foldtext", "formatexpr", "formatlistpat", "formatoptions", "formatprg", "fp", "fs", "fsync", "ft", "gcr", "gd", "gdefault", "gfm", "gfn", "gfs", "gfw", "ghr", "gli", "go", "gp", "grepformat", "grepprg", "gtl", "gtt", "guicursor", "guifont", "guifontset", "guifontwide", "guiheadroom", "guiligatures", "guioptions", "guipty", "guitablabel", "guitabtooltip", "helpfile", "helpheight", "helplang", "hf", "hh", "hi", "hid", "hidden", "highlight", "history", "hk", "hkmap", "hkmapp", "hkp", "hl", "hlg", "hls", "hlsearch", "ic", "icon", "iconstring", "ignorecase", "im", "imactivatefunc", "imactivatekey", "imaf", "imak", "imc", "imcmdline", "imd", "imdisable", "imi", "iminsert", "ims", "imsearch", "imsf", "imst", "imstatusfunc", "imstyle", "inc", "include", "includeexpr", "incsearch", "inde", "indentexpr", "indentkeys", "indk", "inex", "inf", "infercase", "insertmode", "invac", "invacd", "invai", "invallowrevins", "invar", "invarab", "invarabic", "invarabicshape", "invari", "invarshape", "invasd", "invautochdir", "invautocomplete", "invautoindent", "invautoread", "invautoshelldir", "invautowrite", "invautowriteall", "invaw", "invawa", "invbackup", "invballooneval", "invballoonevalterm", "invbeval", "invbevalterm", "invbin", "invbinary", "invbk", "invbl", "invbomb", "invbreakindent", "invbri", "invbuflisted", "invcdh", "invcdhome", "invcf", "invci", "invcin", "invcindent", "invcompatible", "invconfirm", "invcopyindent", "invcp", "invcrb", "invcscoperelative", "invcscopetag", "invcscopeverbose", "invcsre", "invcst", "invcsverb", "invcuc", "invcul", "invcursorbind", "invcursorcolumn", "invcursorline", "invdeco", "invdelcombine", "invdg", "invdiff", "invdigraph", "invea", "inveb", "inved", "invedcompatible", "invek", "invemo", "invemoji", "invendoffile", "invendofline", "inveof", "inveol", "invequalalways", "inverrorbells", "invesckeys", "invet", "invex", "invexpandtab", "invexrc", "invfen", "invfic", "invfileignorecase", "invfixendofline", "invfixeol", "invfoldenable", "invfs", "invfsync", "invgd", "invgdefault", "invguipty", "invhid", "invhidden", "invhk", "invhkmap", "invhkmapp", "invhkp", "invhls", "invhlsearch", "invic", "invicon", "invignorecase", "invim", "invimc", "invimcmdline", "invimd", "invimdisable", "invincsearch", "invinf", "invinfercase", "invinsertmode", "invis", "invjoinspaces", "invjs", "invlangnoremap", "invlangremap", "invlazyredraw", "invlbr", "invlinebreak", "invlisp", "invlist", "invlnr", "invloadplugins", "invlpl", "invlrm", "invlz", "invma", "invmagic", "invmh", "invml", "invmle", "invmod", "invmodeline", "invmodelineexpr", "invmodifiable", "invmodified", "invmore", "invmousef", "invmousefocus", "invmousehide", "invmousemev", "invmousemoveevent", "invnu", "invnumber", "invodev", "invopendevice", "invpaste", "invpi", "invpreserveindent", "invpreviewwindow", "invprompt", "invpvw", "invreadonly", "invrelativenumber", "invremap", "invrestorescreen", "invrevins", "invri", "invrightleft", "invrl", "invrnu", "invro", "invrs", "invru", "invruler", "invsb", "invsc", "invscb", "invscf", "invscrollbind", "invscrollfocus", "invscs", "invsecure", "invsft", "invshellslash", "invshelltemp", "invshiftround", "invshortname", "invshowcmd", "invshowfulltag", "invshowmatch", "invshowmode", "invsi", "invsm", "invsmartcase", "invsmartindent", "invsmarttab", "invsmd", "invsmoothscroll", "invsms", "invsn", "invsol", "invspell", "invsplitbelow", "invsplitright", "invspr", "invsr", "invssl", "invsta", "invstartofline", "invstmp", "invswapfile", "invswf", "invta", "invtagbsearch", "invtagrelative", "invtagstack", "invtbi", "invtbidi", "invtbs", "invtermbidi", "invtermguicolors", "invterse", "invtextauto", "invtextmode", "invtf", "invtgc", "invtgst", "invtildeop", "invtimeout", "invtitle", "invto", "invtop", "invtr", "invttimeout", "invttybuiltin", "invttyfast", "invtx", "invudf", "invundofile", "invvb", "invvisualbell", "invwa", "invwarn", "invwb", "invweirdinvert", "invwfb", "invwfh", "invwfw", "invwic", "invwildignorecase", "invwildmenu", "invwinfixbuf", "invwinfixheight", "invwinfixwidth", "invwiv", "invwlsteal", "invwmnu", "invwrap", "invwrapscan", "invwrite", "invwriteany", "invwritebackup", "invws", "invwst", "invxtermcodes", "is", "isf", "isfname", "isi", "isident", "isk", "iskeyword", "isp", "isprint", "joinspaces", "jop", "js", "jumpoptions", "key", "keymap", "keymodel", "keyprotocol", "keywordprg", "km", "kmp", "kp", "kpc", "langmap", "langmenu", "langnoremap", "langremap", "laststatus", "lazyredraw", "lbr", "lcs", "lhi", "lhistory", "linebreak", "lines", "linespace", "lisp", "lispoptions", "lispwords", "list", "listchars", "lm", "lmap", "lnr", "loadplugins", "lop", "lpl", "lrm", "ls", "lsp", "luadll", "lw", "lz", "ma", "magic", "makeef", "makeencoding", "makeprg", "mat", "matchpairs", "matchtime", "maxcombine", "maxfuncdepth", "maxmapdepth", "maxmem", "maxmempattern", "maxmemtot", "maxsearchcount", "mco", "mef", "menc", "menuitems", "messagesopt", "mfd", "mh", "mis", "mkspellmem", "ml", "mle", "mls", "mm", "mmd", "mmp", "mmt", "mod", "modeline", "modelineexpr", "modelines", "modifiable", "modified", "mopt", "more", "mouse", "mousef", "mousefocus", "mousehide", "mousem", "mousemev", "mousemodel", "mousemoveevent", "mouses", "mouseshape", "mouset", "mousetime", "mp", "mps", "msc", "msm", "mzq", "mzquantum", "mzschemedll", "mzschemegcdll", "nextgroup", "nf", "noac", "noacd", "noai", "noallowrevins", "noar", "noarab", "noarabic", "noarabicshape", "noari", "noarshape", "noasd", "noautochdir", "noautocomplete", "noautoindent", "noautoread", "noautoshelldir", "noautowrite", "noautowriteall", "noaw", "noawa", "nobackup", "noballooneval", "noballoonevalterm", "nobeval", "nobevalterm", "nobin", "nobinary", "nobk", "nobl", "nobomb", "nobreakindent", "nobri", "nobuflisted", "nocdh", "nocdhome", "nocf", "noci", "nocin", "nocindent", "nocompatible", "noconfirm", "nocopyindent", "nocp", "nocrb", "nocscoperelative", "nocscopetag", "nocscopeverbose", "nocsre", "nocst", "nocsverb", "nocuc", "nocul", "nocursorbind", "nocursorcolumn", "nocursorline", "nodeco", "nodelcombine", "nodg", "nodiff", "nodigraph", "noea", "noeb", "noed", "noedcompatible", "noek", "noemo", "noemoji", "noendoffile", "noendofline", "noeof", "noeol", "noequalalways", "noerrorbells", "noesckeys", "noet", "noex", "noexpandtab", "noexrc", "nofen", "nofic", "nofileignorecase", "nofixendofline", "nofixeol", "nofoldenable", "nofs", "nofsync", "nogd", "nogdefault", "noguipty", "nohid", "nohidden", "nohk", "nohkmap", "nohkmapp", "nohkp", "nohls", "nohlsearch", "noic", "noicon", "noignorecase", "noim", "noimc", "noimcmdline", "noimd", "noimdisable", "noincsearch", "noinf", "noinfercase", "noinsertmode", "nois", "nojoinspaces", "nojs", "nolangnoremap", "nolangremap", "nolazyredraw", "nolbr", "nolinebreak", "nolisp", "nolist", "nolnr", "noloadplugins", "nolpl", "nolrm", "nolz", "noma", "nomagic", "nomh", "noml", "nomle", "nomod", "nomodeline", "nomodelineexpr", "nomodifiable", "nomodified", "nomore", "nomousef", "nomousefocus", "nomousehide", "nomousemev", "nomousemoveevent", "nonu", "nonumber", "noodev", "noopendevice", "nopaste", "nopi", "nopreserveindent", "nopreviewwindow", "noprompt", "nopvw", "noreadonly", "norelativenumber", "noremap", "norestorescreen", "norevins", "nori", "norightleft", "norl", "nornu", "noro", "nors", "noru", "noruler", "nosb", "nosc", "noscb", "noscf", "noscrollbind", "noscrollfocus", "noscs", "nosecure", "nosft", "noshellslash", "noshelltemp", "noshiftround", "noshortname", "noshowcmd", "noshowfulltag", "noshowmatch", "noshowmode", "nosi", "nosm", "nosmartcase", "nosmartindent", "nosmarttab", "nosmd", "nosmoothscroll", "nosms", "nosn", "nosol", "nospell", "nosplitbelow", "nosplitright", "nospr", "nosr", "nossl", "nosta", "nostartofline", "nostmp", "noswapfile", "noswf", "nota", "notagbsearch", "notagrelative", "notagstack", "notbi", "notbidi", "notbs", "notermbidi", "notermguicolors", "noterse", "notextauto", "notextmode", "notf", "notgc", "notgst", "notildeop", "notimeout", "notitle", "noto", "notop", "notr", "nottimeout", "nottybuiltin", "nottyfast", "notx", "noudf", "noundofile", "novb", "novisualbell", "nowa", "nowarn", "nowb", "noweirdinvert", "nowfb", "nowfh", "nowfw", "nowic", "nowildignorecase", "nowildmenu", "nowinfixbuf", "nowinfixheight", "nowinfixwidth", "nowiv", "nowlsteal", "nowmnu", "nowrap", "nowrapscan", "nowrite", "nowriteany", "nowritebackup", "nows", "nowst", "noxtermcodes", "nrformats", "nu", "number", "numberwidth", "nuw", "odev", "ofu", "omnifunc", "opendevice", "operatorfunc", "opfunc", "osctimeoutlen", "ost", "pa", "packpath", "para", "paragraphs", "paste", "pastetoggle", "patchexpr", "patchmode", "path", "pb", "pdev", "penc", "perldll", "pex", "pexpr", "pfn", "ph", "pheader", "pi", "pm", "pmbcs", "pmbfn", "pmw", "popt", "pp", "preserveindent", "previewheight", "previewpopup", "previewwindow", "printdevice", "printencoding", "printexpr", "printfont", "printheader", "printmbcharset", "printmbfont", "printoptions", "prompt", "pt", "pumborder", "pumheight", "pummaxwidth", "pumwidth", "pvh", "pvp", "pvw", "pw", "pythondll", "pythonhome", "pythonthreedll", "pythonthreehome", "pyx", "pyxversion", "qe", "qftf", "quickfixtextfunc", "quoteescape", "rdt", "re", "readonly", "redrawtime", "regexpengine", "relativenumber", "remap", "renderoptions", "report", "restorescreen", "revins", "ri", "rightleft", "rightleftcmd", "rl", "rlc", "rnu", "ro", "rop", "rs", "rtp", "ru", "rubydll", "ruf", "ruler", "rulerformat", "runtimepath", "sb", "sbo", "sbr", "sc", "scb", "scf", "scl", "scr", "scroll", "scrollbind", "scrollfocus", "scrolljump", "scrolloff", "scrollopt", "scs", "sect", "sections", "secure", "sel", "selection", "selectmode", "sessionoptions", "sft", "sh", "shcf", "shell", "shellcmdflag", "shellpipe", "shellquote", "shellredir", "shellslash", "shelltemp", "shelltype", "shellxescape", "shellxquote", "shiftround", "shiftwidth", "shm", "shortmess", "shortname", "showbreak", "showcmd", "showcmdloc", "showfulltag", "showmatch", "showmode", "showtabline", "showtabpanel", "shq", "si", "sidescroll", "sidescrolloff", "signcolumn", "siso", "sj", "skipwhite", "slm", "sloc", "sm", "smartcase", "smartindent", "smarttab", "smc", "smd", "smoothscroll", "sms", "sn", "so", "softtabstop", "sol", "sp", "spc", "spell", "spellcapcheck", "spellfile", "spelllang", "spelloptions", "spellsuggest", "spf", "spk", "spl", "splitbelow", "splitkeep", "splitright", "spo", "spr", "sps", "sr", "srr", "ss", "ssl", "ssop", "st", "sta", "stal", "startofline", "statusline", "stl", "stmp", "stpl", "sts", "su", "sua", "suffixes", "suffixesadd", "sw", "swapfile", "swapsync", "swb", "swf", "switchbuf", "sws", "sxe", "sxq", "syn", "synmaxcol", "syntax", "t_8b", "t_8f", "t_8u", "t_AB", "t_AF", "t_AL", "t_AU", "t_BD", "t_BE", "t_CF", "t_CS", "t_CV", "t_Ce", "t_Co", "t_Cs", "t_DL", "t_Ds", "t_EC", "t_EI", "t_F1", "t_F2", "t_F3", "t_F4", "t_F5", "t_F6", "t_F7", "t_F8", "t_F9", "t_GP", "t_IE", "t_IS", "t_K1", "t_K3", "t_K4", "t_K5", "t_K6", "t_K7", "t_K8", "t_K9", "t_KA", "t_KB", "t_KC", "t_KD", "t_KE", "t_KF", "t_KG", "t_KH", "t_KI", "t_KJ", "t_KK", "t_KL", "t_RB", "t_RC", "t_RF", "t_RI", "t_RK", "t_RS", "t_RT", "t_RV", "t_Ri", "t_SC", "t_SH", "t_SI", "t_SR", "t_ST", "t_Sb", "t_Sf", "t_Si", "t_TE", "t_TI", "t_Te", "t_Ts", "t_Us", "t_VS", "t_WP", "t_WS", "t_XM", "t_ZH", "t_ZR", "t_al", "t_bc", "t_cd", "t_ce", "t_cl", "t_cm", "t_cs", "t_da", "t_db", "t_dl", "t_ds", "t_fd", "t_fe", "t_fs", "t_k1", "t_k2", "t_k3", "t_k4", "t_k5", "t_k6", "t_k7", "t_k8", "t_k9", "t_kB", "t_kD", "t_kI", "t_kN", "t_kP", "t_kb", "t_kd", "t_ke", "t_kh", "t_kl", "t_kr", "t_ks", "t_ku", "t_le", "t_mb", "t_md", "t_me", "t_mr", "t_ms", "t_nd", "t_op", "t_se", "t_so", "t_sr", "t_te", "t_ti", "t_ts", "t_u7", "t_ue", "t_us", "t_ut", "t_vb", "t_ve", "t_vi", "t_vs", "t_xn", "t_xo", "t_xs", "ta", "tabclose", "tabline", "tabpagemax", "tabpanel", "tabpanelopt", "tabstop", "tag", "tagbsearch", "tagcase", "tagfunc", "taglength", "tagrelative", "tags", "tagstack", "tal", "tb", "tbi", "tbidi", "tbis", "tbs", "tc", "tcl", "tcldll", "tenc", "term", "termbidi", "termencoding", "termguicolors", "termwinkey", "termwinscroll", "termwinsize", "termwintype", "terse", "textauto", "textmode", "textwidth", "tf", "tfu", "tgc", "tgst", "thesaurus", "thesaurusfunc", "tildeop", "timeout", "timeoutlen", "title", "titlelen", "titleold", "titlestring", "tl", "tm", "to", "toolbar", "toolbariconsize", "top", "tpl", "tplo", "tpm", "tr", "ts", "tsl", "tsr", "tsrfu", "ttimeout", "ttimeoutlen", "ttm", "tty", "ttybuiltin", "ttyfast", "ttym", "ttymouse", "ttyscroll", "ttytype", "tw", "twk", "tws", "twsl", "twt", "tx", "uc", "udf", "udir", "ul", "undodir", "undofile", "undolevels", "undoreload", "updatecount", "updatetime", "ur", "ut", "varsofttabstop", "vartabstop", "vb", "vbs", "vdir", "ve", "verbose", "verbosefile", "vfile", "vi", "viewdir", "viewoptions", "vif", "vimSetEqual", "vimSetMod", "viminfo", "viminfofile", "virtualedit", "visualbell", "vop", "vsts", "vts", "wa", "wak", "warn", "wb", "wc", "wcm", "wcr", "wd", "weirdinvert", "wfb", "wfh", "wfw", "wh", "whichwrap", "wi", "wic", "wig", "wildchar", "wildcharm", "wildignore", "wildignorecase", "wildmenu", "wildmode", "wildoptions", "wim", "winaltkeys", "wincolor", "window", "winfixbuf", "winfixheight", "winfixwidth", "winheight", "winminheight", "winminwidth", "winptydll", "winwidth", "wiv", "wiw", "wlseat", "wlsteal", "wltimeoutlen", "wm", "wmh", "wmnu", "wmw", "wop", "wrap", "wrapmargin", "wrapscan", "write", "writeany", "writebackup", "writedelay", "ws", "wse", "wst", "wtm", "ww", "xtermcodes"]
          kw[:auto] = Set.new ["BufAdd", "BufCreate", "BufDelete", "BufEnter", "BufFilePost", "BufFilePre", "BufHidden", "BufLeave", "BufNew", "BufNewFile", "BufRead", "BufReadCmd", "BufReadPost", "BufReadPre", "BufUnload", "BufWinEnter", "BufWinLeave", "BufWipeout", "BufWrite", "BufWriteCmd", "BufWritePost", "BufWritePre", "CmdUndefined", "CmdlineChanged", "CmdlineEnter", "CmdlineLeave", "CmdlineLeavePre", "CmdwinEnter", "CmdwinLeave", "ColorScheme", "ColorSchemePre", "CompleteChanged", "CompleteDone", "CompleteDonePre", "CursorHold", "CursorHoldI", "CursorMoved", "CursorMovedC", "CursorMovedI", "DiffUpdated", "DirChanged", "DirChangedPre", "EncodingChanged", "ExitPre", "FileAppendCmd", "FileAppendPost", "FileAppendPre", "FileChangedRO", "FileChangedShell", "FileChangedShellPost", "FileEncoding", "FileReadCmd", "FileReadPost", "FileReadPre", "FileType", "FileWriteCmd", "FileWritePost", "FileWritePre", "FilterReadPost", "FilterReadPre", "FilterWritePost", "FilterWritePre", "FocusGained", "FocusLost", "FuncUndefined", "GUIEnter", "GUIFailed", "InsertChange", "InsertCharPre", "InsertEnter", "InsertLeave", "InsertLeavePre", "KeyInputPre", "MenuPopup", "ModeChanged", "OptionSet", "QuickFixCmdPost", "QuickFixCmdPre", "QuitPre", "RemoteReply", "SafeState", "SafeStateAgain", "SessionLoadPost", "SessionWritePost", "ShellCmdPost", "ShellFilterPost", "SigUSR1", "SourceCmd", "SourcePost", "SourcePre", "SpellFileMissing", "StdinReadPost", "StdinReadPre", "SwapExists", "Syntax", "TabClosed", "TabClosedPre", "TabEnter", "TabLeave", "TabNew", "TermChanged", "TermResponse", "TermResponseAll", "TerminalOpen", "TerminalWinOpen", "TextChanged", "TextChangedI", "TextChangedP", "TextChangedT", "TextYankPost", "VimEnter", "VimLeave", "VimLeavePre", "VimResized", "VimResume", "VimSuspend", "WinClosed", "WinEnter", "WinLeave", "WinNew", "WinNewPre", "WinResized", "WinScrolled", "nextgroup", "skipwhite", "vimAutoEventSep", "vimAutocmdPattern"]
          kw[:function] = Set.new ["abs", "acos", "add", "and", "append", "appendbufline", "argc", "argidx", "arglistid", "argv", "asin", "assert_beeps", "assert_equal", "assert_equalfile", "assert_exception", "assert_fails", "assert_false", "assert_inrange", "assert_match", "assert_nobeep", "assert_notequal", "assert_notmatch", "assert_report", "assert_true", "atan", "atan2", "autocmd_add", "autocmd_delete", "autocmd_get", "balloon_gettext", "balloon_show", "balloon_split", "base64_decode", "base64_encode", "bindtextdomain", "blob2list", "blob2str", "browse", "browsedir", "bufadd", "bufexists", "buflisted", "bufload", "bufloaded", "bufname", "bufnr", "bufwinid", "bufwinnr", "byte2line", "byteidx", "byteidxcomp", "call", "ceil", "ch_canread", "ch_close", "ch_close_in", "ch_evalexpr", "ch_evalraw", "ch_getbufnr", "ch_getjob", "ch_info", "ch_log", "ch_logfile", "ch_open", "ch_read", "ch_readblob", "ch_readraw", "ch_sendexpr", "ch_sendraw", "ch_setoptions", "ch_status", "changenr", "char2nr", "charclass", "charcol", "charidx", "chdir", "cindent", "clearmatches", "cmdcomplete_info", "col", "complete", "complete_add", "complete_check", "complete_info", "confirm", "copy", "cos", "cosh", "count", "cscope_connection", "cursor", "debugbreak", "deepcopy", "delete", "deletebufline", "did_filetype", "diff", "diff_filler", "diff_hlID", "digraph_get", "digraph_getlist", "digraph_set", "digraph_setlist", "echoraw", "empty", "environ", "err_teapot", "escape", "eval", "eventhandler", "executable", "execute", "exepath", "exists", "exists_compiled", "exp", "expand", "expandcmd", "extend", "extendnew", "feedkeys", "filecopy", "filereadable", "filewritable", "filter", "finddir", "findfile", "flatten", "flattennew", "float2nr", "floor", "fmod", "fnameescape", "fnamemodify", "foldclosed", "foldclosedend", "foldlevel", "foldtext", "foldtextresult", "foreach", "foreground", "fullcommand", "funcref", "function", "garbagecollect", "get", "getbufinfo", "getbufline", "getbufoneline", "getbufvar", "getcellpixels", "getcellwidths", "getchangelist", "getchar", "getcharmod", "getcharpos", "getcharsearch", "getcharstr", "getcmdcomplpat", "getcmdcompltype", "getcmdline", "getcmdpos", "getcmdprompt", "getcmdscreenpos", "getcmdtype", "getcmdwintype", "getcompletion", "getcompletiontype", "getcurpos", "getcursorcharpos", "getcwd", "getenv", "getfontname", "getfperm", "getfsize", "getftime", "getftype", "getimstatus", "getjumplist", "getline", "getloclist", "getmarklist", "getmatches", "getmousepos", "getmouseshape", "getpid", "getpos", "getqflist", "getreg", "getreginfo", "getregion", "getregionpos", "getregtype", "getscriptinfo", "getstacktrace", "gettabinfo", "gettabvar", "gettabwinvar", "gettagstack", "gettext", "getwininfo", "getwinpos", "getwinposx", "getwinposy", "getwinvar", "glob", "glob2regpat", "globpath", "has", "has_key", "haslocaldir", "hasmapto", "histadd", "histdel", "histget", "histnr", "hlID", "hlexists", "hlget", "hlset", "hostname", "iconv", "id", "indent", "index", "indexof", "input", "inputdialog", "inputlist", "inputrestore", "inputsave", "inputsecret", "insert", "instanceof", "interrupt", "invert", "isabsolutepath", "isdirectory", "isinf", "islocked", "isnan", "items", "job_getchannel", "job_info", "job_setoptions", "job_start", "job_status", "job_stop", "join", "js_decode", "js_encode", "json_decode", "json_encode", "keys", "keytrans", "len", "libcall", "libcallnr", "line", "line2byte", "lispindent", "list2blob", "list2str", "list2tuple", "listener_add", "listener_flush", "listener_remove", "localtime", "log", "log10", "luaeval", "map", "maparg", "mapcheck", "maplist", "mapnew", "mapset", "match", "matchadd", "matchaddpos", "matcharg", "matchbufline", "matchdelete", "matchend", "matchfuzzy", "matchfuzzypos", "matchlist", "matchstr", "matchstrlist", "matchstrpos", "max", "menu_info", "min", "mkdir", "mode", "mzeval", "nextnonblank", "ngettext", "nr2char", "or", "pathshorten", "perleval", "popup_atcursor", "popup_beval", "popup_clear", "popup_close", "popup_create", "popup_dialog", "popup_filter_menu", "popup_filter_yesno", "popup_findecho", "popup_findinfo", "popup_findpreview", "popup_getoptions", "popup_getpos", "popup_hide", "popup_list", "popup_locate", "popup_menu", "popup_move", "popup_notification", "popup_setbuf", "popup_setoptions", "popup_settext", "popup_show", "pow", "preinserted", "prevnonblank", "printf", "prompt_getprompt", "prompt_setcallback", "prompt_setinterrupt", "prompt_setprompt", "prop_add", "prop_add_list", "prop_clear", "prop_find", "prop_list", "prop_remove", "prop_type_add", "prop_type_change", "prop_type_delete", "prop_type_get", "prop_type_list", "pum_getpos", "pumvisible", "py3eval", "pyeval", "pyxeval", "rand", "range", "readblob", "readdir", "readdirex", "readfile", "redraw_listener_add", "redraw_listener_remove", "reduce", "reg_executing", "reg_recording", "reltime", "reltimefloat", "reltimestr", "remote_expr", "remote_foreground", "remote_peek", "remote_read", "remote_send", "remote_startserver", "remove", "rename", "repeat", "resolve", "reverse", "round", "rubyeval", "screenattr", "screenchar", "screenchars", "screencol", "screenpos", "screenrow", "screenstring", "search", "searchcount", "searchdecl", "searchpair", "searchpairpos", "searchpos", "server2client", "serverlist", "setbufline", "setbufvar", "setcellwidths", "setcharpos", "setcharsearch", "setcmdline", "setcmdpos", "setcursorcharpos", "setenv", "setfperm", "setline", "setloclist", "setmatches", "setpos", "setqflist", "setreg", "settabvar", "settabwinvar", "settagstack", "setwinvar", "sha256", "shellescape", "shiftwidth", "sign_define", "sign_getdefined", "sign_getplaced", "sign_jump", "sign_place", "sign_placelist", "sign_undefine", "sign_unplace", "sign_unplacelist", "simplify", "sin", "sinh", "slice", "sort", "sound_clear", "sound_playevent", "sound_playfile", "sound_stop", "soundfold", "spellbadword", "spellsuggest", "split", "sqrt", "srand", "state", "str2blob", "str2float", "str2list", "str2nr", "strcharlen", "strcharpart", "strchars", "strdisplaywidth", "strftime", "strgetchar", "stridx", "string", "strlen", "strpart", "strptime", "strridx", "strtrans", "strutf16len", "strwidth", "submatch", "substitute", "swapfilelist", "swapinfo", "swapname", "synID", "synIDattr", "synIDtrans", "synconcealed", "synstack", "system", "systemlist", "tabpagebuflist", "tabpagenr", "tabpagewinnr", "tagfiles", "taglist", "tan", "tanh", "tempname", "term_dumpdiff", "term_dumpload", "term_dumpwrite", "term_getaltscreen", "term_getansicolors", "term_getattr", "term_getcursor", "term_getjob", "term_getline", "term_getscrolled", "term_getsize", "term_getstatus", "term_gettitle", "term_gettty", "term_list", "term_scrape", "term_sendkeys", "term_setansicolors", "term_setapi", "term_setkill", "term_setrestore", "term_setsize", "term_start", "term_wait", "terminalprops", "test_alloc_fail", "test_autochdir", "test_feedinput", "test_garbagecollect_now", "test_garbagecollect_soon", "test_getvalue", "test_gui_event", "test_ignore_error", "test_mswin_event", "test_null_blob", "test_null_channel", "test_null_dict", "test_null_function", "test_null_job", "test_null_list", "test_null_partial", "test_null_string", "test_null_tuple", "test_option_not_set", "test_override", "test_refcount", "test_setmouse", "test_settime", "test_srand_seed", "test_unknown", "test_void", "timer_info", "timer_pause", "timer_start", "timer_stop", "timer_stopall", "tolower", "toupper", "tr", "trim", "trunc", "tuple2list", "type", "typename", "undofile", "undotree", "uniq", "uri_decode", "uri_encode", "utf16idx", "values", "virtcol", "virtcol2col", "visualmode", "wildmenumode", "wildtrigger", "win_execute", "win_findbuf", "win_getid", "win_gettype", "win_gotoid", "win_id2tabwin", "win_id2win", "win_move_separator", "win_move_statusline", "win_screenpos", "win_splitmove", "winbufnr", "wincol", "windowsversion", "winheight", "winlayout", "winline", "winnr", "winrestcmd", "winrestview", "winsaveview", "winwidth", "wordcount", "writefile", "xor"]
      end
    end
  end
end
