# This file is manually maintained by Michael Huth: michael.huth@byte-physics.de

module Rouge
  module Lexers
    class IgorPro
      # NOTE [jneen]: all elements here must be downcased!

      FUNCTIONS = Set.new %w(
        abs
        acos
        acosh
        addlistitem
        airya
        airyad
        airyb
        airybd
        alog
        annotationinfo
        annotationlist
        area
        areaxy
        asin
        asinh
        atan
        atanh
        atan2
        axisinfo
        axislabel
        axislist
        axisvalfrompixel
        axontelegraphagetdatanum
        axontelegraphagetdatastring
        axontelegraphagetdatastruct
        axontelegraphgetdatanum
        axontelegraphgetdatastring
        axontelegraphgetdatastruct
        axontelegraphgettimeoutms
        axontelegraphsettimeoutms
        base64decode
        base64encode
        besseli
        besselj
        besselk
        bessely
        beta
        betai
        binarysearch
        binarysearchinterp
        binomial
        binomialln
        binomialnoise
        cabs
        capturehistory
        capturehistorystart
        ceil
        centerofmass
        centerofmassxy
        cequal
        char2num
        chebyshev
        chebyshevu
        checkname
        childwindowlist
        cleanupname
        cmplx
        cmpstr
        conj
        contourinfo
        contournamelist
        contournametowaveref
        contourz
        controlnamelist
        converttextencoding
        cos
        cosh
        cosintegral
        cot
        coth
        countobjects
        countobjectsdfr
        cpowi
        createdataobjectname
        creationdate
        csc
        csch
        csrinfo
        csrwave
        csrwaveref
        csrxwave
        csrxwaveref
        ctablist
        datafolderdir
        datafolderexists
        datafolderlist
        datafolderrefchanges
        datafolderrefsequal
        datafolderrefstatus
        date
        datetime
        datetojulian
        date2secs
        dawson
        defined
        deltax
        digamma
        dilogarithm
        dimdelta
        dimoffset
        dimsize
        ei
        elliptice
        elliptick
        enoise
        equalwaves
        erf
        erfc
        erfcw
        erfcx
        exists
        exp
        expint
        expintegrale1
        expnoise
        factorial
        faddeeva
        fakedata
        faverage
        faveragexy
        fdaqmx_ai_channelconfigs
        fdaqmx_ai_getreader
        fdaqmx_ao_updateoutputs
        fdaqmx_connectterminals
        fdaqmx_ctr_finished
        fdaqmx_ctr_isfinished
        fdaqmx_ctr_ispulsefinished
        fdaqmx_ctr_readcounter
        fdaqmx_ctr_readwithoptions
        fdaqmx_ctr_setpulsefrequency
        fdaqmx_ctr_start
        fdaqmx_devicenames
        fdaqmx_dio_finished
        fdaqmx_dio_portwidth
        fdaqmx_dio_read
        fdaqmx_dio_write
        fdaqmx_disconnectterminals
        fdaqmx_errorstring
        fdaqmx_externalcaldate
        fdaqmx_numanaloginputs
        fdaqmx_numanalogoutputs
        fdaqmx_numcounters
        fdaqmx_numdioports
        fdaqmx_readchan
        fdaqmx_readnamedchan
        fdaqmx_resetdevice
        fdaqmx_scangetavailable
        fdaqmx_scangetnextindex
        fdaqmx_scanstart
        fdaqmx_scanstop
        fdaqmx_scanwait
        fdaqmx_scanwaitwithtimeout
        fdaqmx_selfcaldate
        fdaqmx_selfcalibration
        fdaqmx_waveformstart
        fdaqmx_waveformstop
        fdaqmx_wf_isfinished
        fdaqmx_wf_waituntilfinished
        fdaqmx_writechan
        fetchurl
        finddimlabel
        findlistitem
        floor
        fontlist
        fontsizeheight
        fontsizestringwidth
        fresnelcos
        fresnelsin
        funcrefinfo
        functioninfo
        functionlist
        functionpath
        gamma
        gammaeuler
        gammainc
        gammanoise
        gammln
        gammp
        gammq
        gauss
        gauss1d
        gauss2d
        gcd
        geometricmean
        getbrowserline
        getbrowserselection
        getdatafolder
        getdatafolderdfr
        getdefaultfont
        getdefaultfontsize
        getdefaultfontstyle
        getdimlabel
        getenvironmentvariable
        geterrmessage
        getformula
        getindependentmodulename
        getindexedobjname
        getindexedobjnamedfr
        getkeystate
        getrterrmessage
        getrterror
        getrtlocation
        getrtlocinfo
        getrtstackinfo
        getscraptext
        getuserdata
        getwavesdatafolder
        getwavesdatafolderdfr
        getwindowbrowserselection
        gisgetallfileformats
        gissrefsareequal
        gizmoinfo
        gizmoscale
        gnoise
        greplist
        grepstring
        guideinfo
        guidenamelist
        hash
        hcsr
        hdf5attributeinfo
        hdf5datasetinfo
        hdf5libraryinfo
        hdf5linkinfo
        hdf5typeinfo
        hermite
        hermitegauss
        hypergnoise
        hypergpfq
        hyperg0f1
        hyperg1f1
        hyperg2f1
        i
        igorinfo
        igorversion
        imag
        imageinfo
        imagenamelist
        imagenametowaveref
        independentmodulelist
        indexeddir
        indexedfile
        indextoscale
        inf
        integrate1d
        interp
        interp2d
        interp3d
        inverseerf
        inverseerfc
        itemsinlist
        jacobicn
        jacobisn
        juliantodate
        laguerre
        laguerrea
        laguerregauss
        lambertw
        layoutinfo
        leftx
        legendrea
        limit
        listmatch
        listtotextwave
        listtowaverefwave
        ln
        log
        lognormalnoise
        lorentziannoise
        lowerstr
        macroinfo
        macrolist
        macropath
        magsqr
        mandelbrotpoint
        marcumq
        matrixcondition
        matrixdet
        matrixdot
        matrixrank
        matrixtrace
        max
        mcc_autobridgebal
        mcc_autofastcomp
        mcc_autopipetteoffset
        mcc_autoslowcomp
        mcc_autowholecellcomp
        mcc_getbridgebalenable
        mcc_getbridgebalresist
        mcc_getfastcompcap
        mcc_getfastcomptau
        mcc_getholding
        mcc_getholdingenable
        mcc_getmode
        mcc_getneutralizationcap
        mcc_getneutralizationenable
        mcc_getosckillerenable
        mcc_getpipetteoffset
        mcc_getprimarysignalgain
        mcc_getprimarysignalhpf
        mcc_getprimarysignallpf
        mcc_getrscompbandwidth
        mcc_getrscompcorrection
        mcc_getrscompenable
        mcc_getrscompprediction
        mcc_getsecondarysignalgain
        mcc_getsecondarysignallpf
        mcc_getslowcompcap
        mcc_getslowcomptau
        mcc_getslowcomptaux20enable
        mcc_getslowcurrentinjenable
        mcc_getslowcurrentinjlevel
        mcc_getslowcurrentinjsetltime
        mcc_getwholecellcompcap
        mcc_getwholecellcompenable
        mcc_getwholecellcompresist
        mcc_selectmulticlamp700b
        mcc_setbridgebalenable
        mcc_setbridgebalresist
        mcc_setfastcompcap
        mcc_setfastcomptau
        mcc_setholding
        mcc_setholdingenable
        mcc_setmode
        mcc_setneutralizationcap
        mcc_setneutralizationenable
        mcc_setosckillerenable
        mcc_setpipetteoffset
        mcc_setprimarysignalgain
        mcc_setprimarysignalhpf
        mcc_setprimarysignallpf
        mcc_setrscompbandwidth
        mcc_setrscompcorrection
        mcc_setrscompenable
        mcc_setrscompprediction
        mcc_setsecondarysignalgain
        mcc_setsecondarysignallpf
        mcc_setslowcompcap
        mcc_setslowcomptau
        mcc_setslowcomptaux20enable
        mcc_setslowcurrentinjenable
        mcc_setslowcurrentinjlevel
        mcc_setslowcurrentinjsetltime
        mcc_settimeoutms
        mcc_setwholecellcompcap
        mcc_setwholecellcompenable
        mcc_setwholecellcompresist
        mean
        median
        min
        mod
        moddate
        mpfxemgpeak
        mpfxexpconvexppeak
        mpfxgausspeak
        mpfxlorentzianpeak
        mpfxvoigtpeak
        nameofwave
        nan
        newfreedatafolder
        newfreewave
        norm
        normalizeunicode
        note
        numberbykey
        numpnts
        numtype
        numvarordefault
        num2char
        num2istr
        num2str
        nvar_exists
        operationlist
        padstring
        panelresolution
        paramisdefault
        parsefilepath
        pathlist
        pcsr
        pi
        pictinfo
        pictlist
        pixelfromaxisval
        pnt2x
        poissonnoise
        poly
        polygonarea
        poly2d
        possiblyquotename
        proceduretext
        procedureversion
        p2rect
        qcsr
        real
        removebykey
        removeending
        removefromlist
        removelistitem
        replacenumberbykey
        replacestring
        replacestringbykey
        replicatestring
        rightx
        round
        r2polar
        sawtooth
        scaletoindex
        screenresolution
        sec
        sech
        secs2date
        secs2time
        selectnumber
        selectstring
        setenvironmentvariable
        sign
        sin
        sinc
        sinh
        sinintegral
        sortlist
        specialcharacterinfo
        specialcharacterlist
        specialdirpath
        sphericalbessj
        sphericalbessjd
        sphericalbessy
        sphericalbessyd
        sphericalharmonics
        sqlallochandle
        sqlallocstmt
        sqlbinarywavestotextwave
        sqlbindcol
        sqlbindparameter
        sqlbrowseconnect
        sqlbulkoperations
        sqlcancel
        sqlclosecursor
        sqlcolattributenum
        sqlcolattributestr
        sqlcolumnprivileges
        sqlcolumns
        sqlconnect
        sqldatasources
        sqldescribecol
        sqldescribeparam
        sqldisconnect
        sqldriverconnect
        sqldrivers
        sqlendtran
        sqlerror
        sqlexecdirect
        sqlexecute
        sqlfetch
        sqlfetchscroll
        sqlforeignkeys
        sqlfreeconnect
        sqlfreeenv
        sqlfreehandle
        sqlfreestmt
        sqlgetconnectattrnum
        sqlgetconnectattrstr
        sqlgetcursorname
        sqlgetdatanum
        sqlgetdatastr
        sqlgetdescfieldnum
        sqlgetdescfieldstr
        sqlgetdescrec
        sqlgetdiagfieldnum
        sqlgetdiagfieldstr
        sqlgetdiagrec
        sqlgetenvattrnum
        sqlgetenvattrstr
        sqlgetfunctions
        sqlgetinfonum
        sqlgetinfostr
        sqlgetstmtattrnum
        sqlgetstmtattrstr
        sqlgettypeinfo
        sqlmoreresults
        sqlnativesql
        sqlnumparams
        sqlnumresultcols
        sqlnumresultrowsifknown
        sqlnumrowsfetched
        sqlparamdata
        sqlprepare
        sqlprimarykeys
        sqlprocedurecolumns
        sqlprocedures
        sqlputdata
        sqlreinitialize
        sqlrowcount
        sqlsetconnectattrnum
        sqlsetconnectattrstr
        sqlsetcursorname
        sqlsetdescfieldnum
        sqlsetdescfieldstr
        sqlsetdescrec
        sqlsetenvattrnum
        sqlsetenvattrstr
        sqlsetpos
        sqlsetstmtattrnum
        sqlsetstmtattrstr
        sqlspecialcolumns
        sqlstatistics
        sqltableprivileges
        sqltables
        sqltextwavetobinarywaves
        sqltextwaveto2dbinarywave
        sqlupdateboundvalues
        sqlxopcheckstate
        sql2dbinarywavetotextwave
        sqrt
        startmstimer
        statsbetacdf
        statsbetapdf
        statsbinomialcdf
        statsbinomialpdf
        statscauchycdf
        statscauchypdf
        statschicdf
        statschipdf
        statscmssdcdf
        statscorrelation
        statsdexpcdf
        statsdexppdf
        statserlangcdf
        statserlangpdf
        statserrorpdf
        statsevaluecdf
        statsevaluepdf
        statsexpcdf
        statsexppdf
        statsfcdf
        statsfpdf
        statsfriedmancdf
        statsgammacdf
        statsgammapdf
        statsgeometriccdf
        statsgeometricpdf
        statsgevcdf
        statsgevpdf
        statshypergcdf
        statshypergpdf
        statsinvbetacdf
        statsinvbinomialcdf
        statsinvcauchycdf
        statsinvchicdf
        statsinvcmssdcdf
        statsinvdexpcdf
        statsinvevaluecdf
        statsinvexpcdf
        statsinvfcdf
        statsinvfriedmancdf
        statsinvgammacdf
        statsinvgeometriccdf
        statsinvkuipercdf
        statsinvlogisticcdf
        statsinvlognormalcdf
        statsinvmaxwellcdf
        statsinvmoorecdf
        statsinvnbinomialcdf
        statsinvncchicdf
        statsinvncfcdf
        statsinvnormalcdf
        statsinvparetocdf
        statsinvpoissoncdf
        statsinvpowercdf
        statsinvqcdf
        statsinvqpcdf
        statsinvrayleighcdf
        statsinvrectangularcdf
        statsinvspearmancdf
        statsinvstudentcdf
        statsinvtopdowncdf
        statsinvtriangularcdf
        statsinvusquaredcdf
        statsinvvonmisescdf
        statsinvweibullcdf
        statskuipercdf
        statslogisticcdf
        statslogisticpdf
        statslognormalcdf
        statslognormalpdf
        statsmaxwellcdf
        statsmaxwellpdf
        statsmedian
        statsmoorecdf
        statsnbinomialcdf
        statsnbinomialpdf
        statsncchicdf
        statsncchipdf
        statsncfcdf
        statsncfpdf
        statsnctcdf
        statsnctpdf
        statsnormalcdf
        statsnormalpdf
        statsparetocdf
        statsparetopdf
        statspermute
        statspoissoncdf
        statspoissonpdf
        statspowercdf
        statspowernoise
        statspowerpdf
        statsqcdf
        statsqpcdf
        statsrayleighcdf
        statsrayleighpdf
        statsrectangularcdf
        statsrectangularpdf
        statsrunscdf
        statsspearmanrhocdf
        statsstudentcdf
        statsstudentpdf
        statstopdowncdf
        statstriangularcdf
        statstriangularpdf
        statstrimmedmean
        statsusquaredcdf
        statsvonmisescdf
        statsvonmisesnoise
        statsvonmisespdf
        statswaldcdf
        statswaldpdf
        statsweibullcdf
        statsweibullpdf
        stopmstimer
        stringbykey
        stringcrc
        stringfromlist
        stringlist
        stringmatch
        stringtounsignedbytewave
        strlen
        strsearch
        strvarordefault
        str2num
        studenta
        studentt
        sum
        svar_exists
        tableinfo
        tagval
        tagwaveref
        tan
        tanh
        tdmaddchannel
        tdmaddgroup
        tdmappenddatavalues
        tdmappenddatavaluestime
        tdmchannelpropertyexists
        tdmclosechannel
        tdmclosefile
        tdmclosegroup
        tdmcreatechannelproperty
        tdmcreatefile
        tdmcreatefileproperty
        tdmcreategroupproperty
        tdmfilepropertyexists
        tdmgetchannelpropertynames
        tdmgetchannelpropertynum
        tdmgetchannelpropertystr
        tdmgetchannelpropertytime
        tdmgetchannelpropertytype
        tdmgetchannels
        tdmgetchannelstringpropertylen
        tdmgetdatatype
        tdmgetdatavalues
        tdmgetdatavaluestime
        tdmgetfilepropertynames
        tdmgetfilepropertynum
        tdmgetfilepropertystr
        tdmgetfilepropertytime
        tdmgetfilepropertytype
        tdmgetfilestringpropertylen
        tdmgetgrouppropertynames
        tdmgetgrouppropertynum
        tdmgetgrouppropertystr
        tdmgetgrouppropertytime
        tdmgetgrouppropertytype
        tdmgetgroups
        tdmgetgroupstringpropertylen
        tdmgetlibraryerrordescription
        tdmgetnumchannelproperties
        tdmgetnumchannels
        tdmgetnumdatavalues
        tdmgetnumfileproperties
        tdmgetnumgroupproperties
        tdmgetnumgroups
        tdmgrouppropertyexists
        tdmopenfile
        tdmopenfileex
        tdmremovechannel
        tdmremovegroup
        tdmreplacedatavalues
        tdmreplacedatavaluestime
        tdmsavefile
        tdmsetchannelpropertynum
        tdmsetchannelpropertystr
        tdmsetchannelpropertytime
        tdmsetdatavalues
        tdmsetdatavaluestime
        tdmsetfilepropertynum
        tdmsetfilepropertystr
        tdmsetfilepropertytime
        tdmsetgrouppropertynum
        tdmsetgrouppropertystr
        tdmsetgrouppropertytime
        textencodingcode
        textencodingname
        textfile
        threadgroupcreate
        threadgroupgetdf
        threadgroupgetdfr
        threadgrouprelease
        threadgroupwait
        threadprocessorcount
        threadreturnvalue
        ticks
        time
        tracefrompixel
        traceinfo
        tracenamelist
        tracenametowaveref
        trimstring
        trunc
        uniquename
        unpadstring
        unsetenvironmentvariable
        upperstr
        urldecode
        urlencode
        variablelist
        variance
        vcsr
        viassertintrsignal
        viasserttrigger
        viassertutilsignal
        viclear
        viclose
        vidisableevent
        vidiscardevents
        vienableevent
        vifindnext
        vifindrsrc
        vigetattribute
        vigetattributestring
        vigpibcommand
        vigpibcontrolatn
        vigpibcontrolren
        vigpibpasscontrol
        vigpibsendifc
        viin8
        viin16
        viin32
        vilock
        vimapaddress
        vimaptrigger
        vimemalloc
        vimemfree
        vimovein8
        vimovein16
        vimovein32
        vimoveout8
        vimoveout16
        vimoveout32
        viopen
        viopendefaultrm
        viout8
        viout16
        viout32
        vipeek8
        vipeek16
        vipeek32
        vipoke8
        vipoke16
        vipoke32
        viread
        vireadstb
        visetattribute
        visetattributestring
        vistatusdesc
        viterminate
        viunlock
        viunmapaddress
        viunmaptrigger
        viusbcontrolin
        viusbcontrolout
        vivxicommandquery
        viwaitonevent
        viwrite
        voigtfunc
        voigtpeak
        wavecrc
        wavedatatostring
        wavedims
        waveexists
        wavehash
        waveinfo
        wavelist
        wavemax
        wavemin
        waveminandmax
        wavemodcount
        wavename
        waverefindexed
        waverefindexeddfr
        waverefsequal
        waverefwavetolist
        wavetextencoding
        wavetype
        waveunits
        whichlistitem
        winlist
        winname
        winrecreation
        wintype
        wnoise
        xcsr
        xwavename
        xwavereffromtrace
        x2pnt
        zcsr
        zerniker
        zeromq_client_connect
        zeromq_client_recv
        zeromq_client_send
        zeromq_handler_start
        zeromq_handler_stop
        zeromq_pub_bind
        zeromq_pub_send
        zeromq_server_bind
        zeromq_server_recv
        zeromq_server_send
        zeromq_set
        zeromq_set_logging_template
        zeromq_stop
        zeromq_sub_add_filter
        zeromq_sub_connect
        zeromq_sub_recv
        zeromq_sub_remove_filter
        zeromq_test_callfunction
        zeromq_test_serializewave
        zeta
      )

      OPERATIONS = Set.new %w(
        abort
        addfifodata
        addfifovectdata
        addmovieaudio
        addmovieframe
        addwavestoboxplot
        addwavestoviolinplot
        adoptfiles
        apmath
        append
        appendboxplot
        appendimage
        appendlayoutobject
        appendmatrixcontour
        appendtext
        appendtogizmo
        appendtograph
        appendtolayout
        appendtotable
        appendviolinplot
        appendxyzcontour
        autopositionwindow
        axontelegraphfindservers
        backgroundinfo
        beep
        beziertopolygon
        boundingball
        boxsmooth
        browseurl
        buildmenu
        button
        cd
        chart
        checkbox
        checkdisplayed
        choosecolor
        close
        closehelp
        closemovie
        closeproc
        colorscale
        colortab2wave
        concatenate
        controlbar
        controlinfo
        controlupdate
        convertglobalstringtextencoding
        convexhull
        convolve
        copydimlabels
        copyfile
        copyfolder
        copyscales
        correlate
        createaliasshortcut
        createbrowser
        cross
        ctrlbackground
        ctrlfifo
        ctrlnamedbackground
        cursor
        curvefit
        customcontrol
        cwt
        daqmx_ai_setupreader
        daqmx_ao_setoutputs
        daqmx_ctr_countedges
        daqmx_ctr_outputpulse
        daqmx_ctr_period
        daqmx_ctr_pulsewidth
        daqmx_deviceinfo
        daqmx_dio_config
        daqmx_dio_writenewdata
        daqmx_scan
        daqmx_waveformgen
        debugger
        debuggeroptions
        defaultfont
        defaultguicontrols
        defaultguifont
        defaulttextencoding
        defineguide
        delayupdate
        deleteannotations
        deletefile
        deletefolder
        deletepoints
        differentiate
        dir
        display
        displayhelptopic
        displayprocedure
        doalert
        doigormenu
        doupdate
        dowindow
        doxopidle
        dpss
        drawaction
        drawarc
        drawbezier
        drawline
        drawoval
        drawpict
        drawpoly
        drawrect
        drawrrect
        drawtext
        drawusershape
        dspdetrend
        dspperiodogram
        duplicate
        duplicatedatafolder
        dwt
        edgestats
        edit
        errorbars
        estimatepeaksizes
        execute
        executescripttext
        experimentinfo
        experimentmodified
        exportgizmo
        extract
        fastgausstransform
        fastop
        fbinread
        fbinwrite
        fcall_callfunction
        fcall_freelibrary
        fcall_getfunctionlist
        fcall_getparamtypelist
        fcall_loadlibrary
        fcall_version
        fft
        fgetpos
        fifostatus
        fifo2wave
        filterfir
        filteriir
        findapeak
        findcontour
        findduplicates
        findlevel
        findlevels
        findpeak
        findpointsinpoly
        findroots
        findsequence
        findvalue
        fmaxflat
        fpclustering
        fprintf
        freadline
        fsetpos
        fstatus
        ftpcreatedirectory
        ftpdelete
        ftpdownload
        ftpupload
        funcfit
        funcfitmd
        gbloadwave
        getaxis
        getcamera
        getfilefolderinfo
        getgizmo
        getlastusermenuinfo
        getmarquee
        getmouse
        getselection
        getwindow
        giscreatevectorlayer
        gisgetrasterinfo
        gisgetregisteredfileinfo
        gisgetvectorlayerinfo
        gisloadrasterdata
        gisloadvectordata
        gisrasterizevectordata
        gisregisterfile
        gistransformcoords
        gisunregisterfile
        giswritefielddata
        giswritegeometrydata
        giswriteraster
        gpibreadbinarywave2
        gpibreadbinary2
        gpibreadwave2
        gpibread2
        gpibwritebinarywave2
        gpibwritebinary2
        gpibwritewave2
        gpibwrite2
        gpib2
        graphnormal
        graphwavedraw
        graphwaveedit
        grep
        groupbox
        hanning
        hcluster
        hdfinfo
        hdfreadimage
        hdfreadsds
        hdfreadvset
        hdf5closefile
        hdf5closegroup
        hdf5control
        hdf5createfile
        hdf5creategroup
        hdf5createlink
        hdf5dimensionscale
        hdf5dump
        hdf5dumperrors
        hdf5flushfile
        hdf5listattributes
        hdf5listgroup
        hdf5loaddata
        hdf5loadgroup
        hdf5loadimage
        hdf5openfile
        hdf5opengroup
        hdf5savedata
        hdf5savegroup
        hdf5saveimage
        hdf5unlinkobject
        hideigormenus
        hideinfo
        hideprocedures
        hidetools
        hilberttransform
        histogram
        ica
        ifft
        imageanalyzeparticles
        imageblend
        imageboundarytomask
        imagecomposite
        imageedgedetection
        imagefileinfo
        imagefilter
        imagefocus
        imagefromxyz
        imagegenerateroimask
        imageglcm
        imagehistmodification
        imagehistogram
        imageinterpolate
        imagelineprofile
        imageload
        imagemorphology
        imageregistration
        imageremovebackground
        imagerestore
        imagerotate
        imagesave
        imageseedfill
        imageskeleton3d
        imagesnake
        imagestats
        imagethreshold
        imagetransform
        imageunwrapphase
        imagewindow
        indexsort
        insertpoints
        instantfrequency
        integrate
        integrateode
        integrate2d
        interpolate2
        interpolate3d
        interp3dpath
        itccloseall2
        itcclosedevice2
        itcconfigallchannels2
        itcconfigchannelreset2
        itcconfigchannelupload2
        itcconfigchannel2
        itcfifoavailableall2
        itcfifoavailable2
        itcgetallchannelsconfig2
        itcgetchannelconfig2
        itcgetcurrentdevice2
        itcgetdeviceinfo2
        itcgetdevices2
        itcgeterrorstring2
        itcgetserialnumber2
        itcgetstate2
        itcgetversions2
        itcinitialize2
        itcopendevice2
        itcreadadc2
        itcreaddigital2
        itcreadtimer2
        itcselectdevice2
        itcsetdac2
        itcsetglobals2
        itcsetmodes2
        itcsetstate2
        itcstartacq2
        itcstopacq2
        itcupdatefifopositionall2
        itcupdatefifoposition2
        itcwritedigital2
        jcamploadwave
        jointhistogram
        jsonxop_addtree
        jsonxop_addvalue
        jsonxop_dump
        jsonxop_getarraysize
        jsonxop_getkeys
        jsonxop_getmaxarraysize
        jsonxop_gettype
        jsonxop_getvalue
        jsonxop_new
        jsonxop_parse
        jsonxop_release
        jsonxop_remove
        jsonxop_version
        killbackground
        killcontrol
        killdatafolder
        killfifo
        killfreeaxis
        killpath
        killpicts
        killstrings
        killvariables
        killwaves
        killwindow
        kmeans
        label
        layout
        layoutpageaction
        layoutslideshow
        legend
        linearfeedbackshiftregister
        listbox
        loaddata
        loadpackagepreferences
        loadpict
        loadwave
        loess
        lombperiodogram
        make
        makeindex
        markperftesttime
        matrixbalance
        matrixconvolve
        matrixcorr
        matrixeigenv
        matrixfactor
        matrixfilter
        matrixgaussj
        matrixglm
        matrixinverse
        matrixlinearsolve
        matrixlinearsolvetd
        matrixlls
        matrixlubksub
        matrixlud
        matrixludtd
        matrixmultiply
        matrixmultiplyadd
        matrixop
        matrixreversebalance
        matrixschur
        matrixsolve
        matrixsparse
        matrixsvbksub
        matrixsvd
        matrixtranspose
        mcc_findservers
        measurestyledtext
        mfr_checkfornewbricklets
        mfr_closeresultfile
        mfr_createoverviewtable
        mfr_getbrickletcount
        mfr_getbrickletdata
        mfr_getbrickletdeploydata
        mfr_getbrickletmetadata
        mfr_getbrickletrawdata
        mfr_getreporttemplate
        mfr_getresultfilemetadata
        mfr_getresultfilename
        mfr_getvernissageversion
        mfr_getversion
        mfr_getxoperrormessage
        mfr_openresultfile
        mlloadwave
        modify
        modifyboxplot
        modifybrowser
        modifycamera
        modifycontour
        modifycontrol
        modifycontrollist
        modifyfreeaxis
        modifygizmo
        modifygraph
        modifyimage
        modifylayout
        modifypanel
        modifyprocedure
        modifytable
        modifyviolinplot
        modifywaterfall
        movedatafolder
        movefile
        movefolder
        movestring
        movesubwindow
        movevariable
        movewave
        movewindow
        multitaperpsd
        multithreadingcontrol
        nc_closefile
        nc_dumperrors
        nc_inquire
        nc_listattributes
        nc_listobjects
        nc_loaddata
        nc_openfile
        neuralnetworkrun
        neuralnetworktrain
        newcamera
        newdatafolder
        newfifo
        newfifochan
        newfreeaxis
        newgizmo
        newimage
        newlayout
        newmovie
        newnotebook
        newpanel
        newpath
        newwaterfall
        niloadwave
        ni4882
        note
        notebook
        notebookaction
        open
        openhelp
        opennotebook
        optimize
        parseoperationtemplate
        pathinfo
        pauseforuser
        pauseupdate
        pca
        playmovie
        playmovieaction
        playsound
        polygonop
        popupcontextualmenu
        popupmenu
        preferences
        primefactors
        print
        printf
        printgraphs
        printlayout
        printnotebook
        printsettings
        printtable
        project
        pulsestats
        putscraptext
        pwd
        quit
        ratiofromnumber
        redimension
        remez
        remove
        removecontour
        removefromgizmo
        removefromgraph
        removefromlayout
        removefromtable
        removeimage
        removelayoutobjects
        removepath
        rename
        renamedatafolder
        renamepath
        renamepict
        renamewindow
        reorderimages
        reordertraces
        replacetext
        replacewave
        resample
        resumeupdate
        reverse
        rotate
        save
        savedata
        saveexperiment
        savegizmocopy
        savegraphcopy
        savenotebook
        savepackagepreferences
        savepict
        savetablecopy
        setactivesubwindow
        setaxis
        setbackground
        setdashpattern
        setdatafolder
        setdimlabel
        setdrawenv
        setdrawlayer
        setfilefolderinfo
        setformula
        setidleperiod
        setigorhook
        setigormenumode
        setigoroption
        setmarquee
        setprocesssleep
        setrandomseed
        setscale
        setvariable
        setwavelock
        setwavetextencoding
        setwindow
        showigormenus
        showinfo
        showtools
        silent
        sleep
        slider
        smooth
        smoothcustom
        sort
        sortcolumns
        soundinrecord
        soundinset
        soundinstartchart
        soundinstatus
        soundinstopchart
        soundloadwave
        soundsavewave
        sphericalinterpolate
        sphericaltriangulate
        splitstring
        splitwave
        sprintf
        sqlhighlevelop
        sscanf
        stack
        stackwindows
        statsangulardistancetest
        statsanova1test
        statsanova2nrtest
        statsanova2rmtest
        statsanova2test
        statschitest
        statscircularcorrelationtest
        statscircularmeans
        statscircularmoments
        statscirculartwosampletest
        statscochrantest
        statscontingencytable
        statsdiptest
        statsdunnetttest
        statsfriedmantest
        statsftest
        statshodgesajnetest
        statsjbtest
        statskde
        statskendalltautest
        statskstest
        statskwtest
        statslinearcorrelationtest
        statslinearregression
        statsmulticorrelationtest
        statsnpmctest
        statsnpnominalsrtest
        statsquantiles
        statsrankcorrelationtest
        statsresample
        statssample
        statsscheffetest
        statsshapirowilktest
        statssigntest
        statssrtest
        statsttest
        statstukeytest
        statsvariancestest
        statswatsonusquaredtest
        statswatsonwilliamstest
        statswheelerwatsontest
        statswilcoxonranktest
        statswrcorrelationtest
        stft
        structfill
        structget
        structput
        sumdimension
        sumseries
        tabcontrol
        tag
        tdmloaddata
        tdmsavedata
        textbox
        texthistogram
        text2bezier
        threadgroupputdf
        threadstart
        tickwavesfromaxis
        tile
        tilewindows
        titlebox
        tocommandline
        toolsgrid
        triangulate3d
        tufxop_acquirelock
        tufxop_clear
        tufxop_getstorage
        tufxop_init
        tufxop_releaselock
        tufxop_runninginmainthread
        tufxop_version
        unwrap
        unzipfile
        urlrequest
        valdisplay
        vdtcloseport2
        vdtgetportlist2
        vdtgetstatus2
        vdtopenport2
        vdtoperationsport2
        vdtreadbinarywave2
        vdtreadbinary2
        vdtreadhexwave2
        vdtreadhex2
        vdtreadwave2
        vdtread2
        vdtterminalport2
        vdtwritebinarywave2
        vdtwritebinary2
        vdtwritehexwave2
        vdtwritehex2
        vdtwritewave2
        vdtwrite2
        vdt2
        visacontrol
        visaread
        visareadbinary
        visareadbinarywave
        visareadwave
        visawrite
        visawritebinary
        visawritebinarywave
        visawritewave
        wavemeanstdv
        wavestats
        wavetracking
        wavetransform
        wfprintf
        wignertransform
        windowfunction
        xlloadwave
      )
    end
  end
end
